@@ -19,7 +19,7 @@ use Fcntl qw/SEEK_END/;
 use English qw/ -no_match_vars /;
 use base qw/Exporter/;
 
-our $VERSION     = version->new('0.2.0');
+our $VERSION     = version->new('0.3.0');
 
 Readonly my @LOG_LEVELS => qw/info message debug warn error fatal/;
 
@@ -140,7 +140,10 @@ sub error {
 
 	$params[0]{-level} = 'error';
 
-	return $self->record(@params);
+	my $ans = $self->record(@params);
+	$self->flush;
+
+	return $ans;
 }
 
 sub fatal {
@@ -240,17 +243,17 @@ sub log_handle {
 
 		# guarentee that there is a new line before we start writing
 		my $missing = 0;
-		if ( -s $file ) {
+		if ( !$self->{reopening} && -s $file ) {
 			open my $fh, '<', $file or die "Could not open the log file $file to check that it ends in a new line: $OS_ERROR\n";
-			seek $fh, -2, SEEK_END;
+			seek $fh, -20, SEEK_END;
 			my $end = <$fh>;
 			$missing = $end =~ /\n$/;
 			close $fh;
 		}
 
 		open my $fh, '>>', $file or die "Could not open log file $file: $!\n";
-		$self->{log_file} = $file;
-		$self->{handle}   = $fh;
+		$self->{file}   = $file;
+		$self->{handle} = $fh;
 
 		if ($missing) {
 			print {$fh} "\n";
@@ -360,7 +363,7 @@ sub is_security { return 1                        }
 sub file {
 	my ($self) = @_;
 
-	return $self->{log_file};
+	return $self->{file};
 }
 
 sub catch_warnings {
@@ -389,6 +392,18 @@ sub catch_warnings {
 	return $self->{old_warn_handle} && 1;
 }
 
+sub flush {
+	my ($self) = @_;
+
+	return if ! exists $self->{handle};
+
+	close $self->{handle};
+	delete $self->{handle};
+	$self->{reopening} = 1;
+
+	return;
+}
+
 sub DESTROY {
 	my ($self) = @_;
 
@@ -409,7 +424,7 @@ Log::Deep - Deep Logging of information about a script state
 
 =head1 VERSION
 
-This documentation refers to Log::Deep version 0.2.0.
+This documentation refers to Log::Deep version 0.3.0.
 
 
 =head1 SYNOPSIS
@@ -634,6 +649,10 @@ Return: bool - True if currently catching warnings, false if not
 
 Description: Turns on/off catching warnings and/or returns the current warn catching state.
 
+=head3 C<flush ()>
+
+Description: Calls IO::Handle's flush on the log file handle
+
 =head1 DIAGNOSTICS
 
 =head1 CONFIGURATION AND ENVIRONMENT
